'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fetch = require('node-fetch');
var convenienceFunctions = require('@hbauer/convenience-functions');
var fs = require('fs');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fetch__default = /*#__PURE__*/_interopDefaultLegacy(fetch);

/** @param {string} path */
const loadWords = path => fs.promises.readFile(path, 'utf-8');

/** @param {string} str */
const splitLines = str => str.split('\n');

/** @param {string[]} words */
const selectWord = words =>
  words[Math.floor(Math.random() * words.length)];

/**
 * @param {string} targetDomain
 * @returns {Promise<string[]>}
 */
function getInstanceNames(targetDomain) {
  const url = convenienceFunctions.buildURL({
    host: 'https://porkbun.com',
    path: `api/json/v3/dns/retrieve/${targetDomain}`,
  });

  /**
   * @param {{ records: { name: string, type: string }[] }} param0
   * @returns
   */
  const transform = ({ records }) =>
    records
      .filter(record => record.type === 'A')
      .map(record => record.name.split('.')[0]);

  const body = {
    secretapikey: process.env.API_SECRET,
    apikey: process.env.API_KEY,
  };

  const init = {
    method: 'post',
    headers: { 'content-type': 'application/json' },
    body: JSON.stringify(body),
  };

  // @ts-ignore
  return fetch__default["default"](url, init)
    .then(response => response.json())
    .then(transform)
}

/**
 * Return a unique subdomain name
 * @param {string} targetDomain
 * @param {string} path
 * @returns {Promise<string>}
 */
const getSubdomain = async (targetDomain, path) => {
  const name = await loadWords(path).then(splitLines).then(selectWord);
  const names = await getInstanceNames(targetDomain);
  return names.includes(name) ? getSubdomain(path, targetDomain) : name
};

/**
 *
 * @param {string} targetDomain
 * @param {string} subdomain
 * @returns {Promise<{ status: string, message: string }>}
 */
function createAddressRecord(targetDomain, subdomain) {
  const url = convenienceFunctions.buildURL({
    host: 'https://porkbun.com',
    path: `api/json/v3/dns/create/${targetDomain}`,
  });

  const body = {
    secretapikey: process.env.API_SECRET,
    apikey: process.env.API_KEY,
    name: subdomain,
    type: 'A',
    content: process.env.INSTANCE_IP,
  };

  const init = {
    method: 'post',
    headers: { 'content-type': 'application/json' },
    body: JSON.stringify(body),
  };

  // @ts-ignore
  return fetch__default["default"](url, init).then(response => response.json())
}

exports.createAddressRecord = createAddressRecord;
exports.getSubdomain = getSubdomain;
